unit sashellctrls;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Controls, ComCtrls;

type

  TShellListView = class(TListView)
  private
    fPath: string;
    fMultiPath: Boolean; //nur bei multi pfad listviews, z.b. für Programme-Ordner
  protected
    procedure SetParent(NewParent: TWinControl); override;
    procedure Resize; override;
    procedure AdjustColWidth; //Spaltenbreiten wird von Resize aufgerufen
    procedure CreateColumns;
    procedure ClearList;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure GetFiles(ADir : string);
    procedure ParenFolder;
    property Path : string read fPath;
  end;

implementation

uses
  {$ifdef windows}lfmosfuncwin,{$else}lfmosfuncnix,{$endif}
  FileUtil;

const
  //dateispalten in Listview. Darf nicht so einfach geändert werden siehe GetFiles(ADir : string);
  coFName = 0;
  coFType = 1;
  coFSize = 2;
  coFDate = 3;
  coFAttr = 4;
  coFPath = 5;



constructor TShellListView.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  ReadOnly:=true;

  CreateColumns;

  fPath:='';
end;

destructor TShellListView.Destroy;
begin
  inherited Destroy;
end;

procedure TShellListView.SetParent(NewParent: TWinControl);
begin
   inherited SetParent(NewParent);
   SetListViewImgListToSys(self, true);//ListView_SetImageList(Handle,GetSysImageListHandle(false),SHGFI_SMALLICON);
   SetListViewImgListToSys(self, false);//small
   //AkFView.GetFiles(ExtractFilePath(ParamStr(0)));
end;

procedure TShellListView.Resize;
begin
  inherited Resize;

  if ViewStyle=vsReport then AdjustColWidth; //Spaltenbreiten
end;

procedure TShellListView.AdjustColWidth; //Spaltenbreiten wird von Resize aufgerufen
begin
  //if ViewStyle=vsReport then
  // 30,10,15,20
  Column[coFName].Width:=(30 * ClientWidth) div 100;
  Column[coFType].Width:=(15 * ClientWidth) div 100;
  Column[coFSize].Width:=(15 * ClientWidth) div 100;
  Column[coFDate].Width:=(25 * ClientWidth) div 100;
  Column[coFAttr].Width:=(10 * ClientWidth) div 100;
  //rest path
  Column[coFPath].Width:=(4 * ClientWidth) div 100;
end;

procedure TShellListView.CreateColumns;
begin
  Self.Columns.Add;
  Self.Columns.Add;
  Self.Columns.Add;
  Self.Columns.Add;
  Self.Columns.Add;
  Self.Columns.Add;
  Self.Column[coFName].Caption := 'Name';
  Self.Column[coFType].Caption := 'Type';
  Self.Column[coFSize].Caption := 'Size';  Self.Column[coFSize].Alignment:=taRightJustify; //soner
  Self.Column[coFDate].Caption := 'Datum';
  Self.Column[coFAttr].Caption := 'Attr';
  Self.Column[coFPath].Caption := 'Pfad';
end;

procedure TShellListView.ClearList;
begin
  Items.Clear;
end;

procedure TShellListView.GetFiles(ADir : string);
var aSr: TSearchRec;
    aExt, aAttrStr: string;
    aFileAttrs: Integer;
    aItem: TListItem;
begin
  if (ADir='') then exit;
  Items.BeginUpdate;
  ClearList; //erst alles entfernen

  if ADir[Length(ADir)]<>PathDelim then ADir:=ADir+PathDelim;
  //test
  aFileAttrs:= faHidden + faSysFile + faDirectory + faVolumeId + faArchive + faReadOnly + faSymLink +faAnyFile;
  //

 try
  if FindFirstUTF8(ADir+'*.*', aFileAttrs, aSr) = 0 then begin
    repeat
      if (aSr.Name<>'.')and(aSr.Name<>'..') then begin
        //um suche zu vereinfachen: aExt:=AnsiLowerCase(ExtractFileExt(aSr.Name)+';'); //z.b. .htm;
        aAttrStr:='';

        if (aSr.Attr and faDirectory) = faDirectory then aAttrStr:=aAttrStr+'D';
        if (aSr.Attr and faHidden) = faHidden then aAttrStr:=aAttrStr+'H';
        if (aSr.Attr and faReadOnly) = faReadOnly then aAttrStr:=aAttrStr+'R';
        if (aSr.Attr and faArchive) = faArchive then aAttrStr:=aAttrStr+'A';
        if (aSr.Attr and faSysFile) = faSysFile then aAttrStr:=aAttrStr+'S';
        if (aSr.Attr and faSymLink) = faSymLink then aAttrStr:=aAttrStr+'L';
        if (aSr.Attr and faVolumeId) = faVolumeId then aAttrStr:=aAttrStr+'V';
       { if (aSr.Attr and faDirectory) = faDirectory then begin

          //if SubDirRecursive then  aSearchDir(ADir+aSr.Name+'\');
          // else aFiles:=aFiles+ADir+aSr.Name+#0;
        end
        else begin
          //aFiles:=aFiles+ADir+aSr.Name+#0;
        end;  }
        //test
        aItem:=Items.Add;
        aItem.Caption:=aSr.Name;
        aItem.SubItems.Add(ExtractFileExt(aSr.Name)); //type
        aItem.SubItems.Add(IntToStr(aSr.Size)); //size
        aItem.SubItems.Add(DateTimeToStr(FileDateToDateTime(aSr.Time))); //funkt nicht: TimeToStr(aSr.Time));
        aItem.SubItems.Add(aAttrStr); //atrribute
        if fMultiPath then aItem.SubItems.Add(ADir) //pfad nur bei multi pfad listviews
         else aItem.SubItems.Add('');

        aItem.ImageIndex:=GetFileSysImageIndex(ADir+aSr.Name{$ifndef windows},aExt{$endif}); //soner test:  NewItem.ImageIndex:=5;
        //Overlayindex and sharedindex
        SetListViewOverlayIndex(self, aItem.Index,GetListViewOverlayIndex(ADir+aSr.Name)); //soner


        {New(aFRec);
        aFRec^.Name:=aSr.Name;
        aFRec^.Time:=aSr.Time;
        aFRec^.Attr:=aSr.Attr;
        aFRec^.ExcludeAttr:=aSr.ExcludeAttr;
        aFRec^.Size:=aSr.Size;
        aFRec^.ImgIndex:=GetFileSysImageIndex(ADir+aSr.Name);
        AFList.Add(aFRec);}
        //ende test
      end;
    until FindNextUTF8(aSr) <> 0;

  end;  //von if FindFirst(

 finally
  FindCloseUTF8(aSr);
  fPath:=ADir;
  Items.EndUpdate;
 end;
end;

procedure TShellListView.ParenFolder;
var aPath: string;
begin
  if fPath='' then exit;
  aPath:=ExtractFilePath(ExtractFileDir(fPath));
  GetFiles(aPath);
end;

end.
